@@ -23,6 +23,7 @@
  */
 
 #define _GNU_SOURCE
+#define USE_LIBDC1394 0
 
 #include <stdio.h>
 #include <stdint.h>
@@ -625,7 +626,7 @@ chameleon_get_image_size_from_video_mode(struct chameleon_camera *camera, dc1394
     return DC1394_FAILURE;
 }
 
-static dc1394error_t
+dc1394error_t
 chameleon_video_get_framerate(struct chameleon_camera *camera, dc1394framerate_t *framerate)
 {
     uint32_t value;
@@ -1333,7 +1334,7 @@ chameleon_feature_get_value(struct chameleon_camera *camera, dc1394feature_t fea
 
     FEATURE_TO_VALUE_OFFSET(feature, offset);
 
-    printf("fetching from register 0x%x\n", offset);
+    printf("fetching from register 0x%lx\n", (unsigned long)offset);
 
     err=chameleon_get_control_register(camera, offset, &quadval);
     DC1394_ERR_RTN(err, "Could not get feature value");
