@@ -56,7 +56,7 @@ static PyObject *ScannerError;
 #define MIN(a,b) ((a)<(b)?(a):(b))
 #define MAX(a,b) ((a)>(b)?(a):(b))
 
-#define MAX_REGIONS 2000
+#define MAX_REGIONS 4000
 
 struct scan_params {
     uint16_t min_region_area;
@@ -384,6 +384,12 @@ static void quantise_image(const struct bgr *in,
 	}
 }
 
+static bool is_zero_bgr(const struct bgr *v)
+{
+	return v->r == 0 && v->g == 0 && v->b == 0;
+}
+
+
 /*
   unquantise an BGR image, useful for visualising the effect of
   quantisation by restoring the original colour ranges, which makes
@@ -399,9 +405,13 @@ static void unquantise_image(const struct bgr_image *in,
 	for (y=0; y<in->height; y++) {
 		for (x=0; x<in->width; x++) {
 			const struct bgr *v = &in->data[y][x];
-			out->data[y][x].r = (v->r * bin_spacing->r) + min->r;
-			out->data[y][x].g = (v->g * bin_spacing->g) + min->g;
-			out->data[y][x].b = (v->b * bin_spacing->b) + min->b;
+			if (is_zero_bgr(v)) {
+                            out->data[y][x] = *v;
+                        } else {
+                            out->data[y][x].r = (v->r * bin_spacing->r) + min->r;
+                            out->data[y][x].g = (v->g * bin_spacing->g) + min->g;
+                            out->data[y][x].b = (v->b * bin_spacing->b) + min->b;
+                        }
 		}
 	}
 
@@ -593,11 +603,6 @@ static void colour_histogram(const struct scan_params *scan_params,
 #define REGION_UNKNOWN -2
 #define REGION_NONE -1
 
-static bool is_zero_bgr(const struct bgr *v)
-{
-	return v->r == 0 && v->g == 0 && v->b == 0;
-}
-
 /*
   find a region number for a pixel by looking at the surrounding pixels
   up to scan_params.region_merge
@@ -1121,6 +1126,7 @@ scanner_scan(PyObject *self, PyObject *args)
 
         const struct bgr_image *in = allocate_bgr_image8(height, width, PyArray_DATA(img_in));
 
+
 	struct regions *regions = any_matrix(2, 
                                              sizeof(int16_t), 
                                              offsetof(struct regions, data), 
@@ -1165,7 +1171,23 @@ scanner_scan(PyObject *self, PyObject *args)
         }
 
 	prune_large_regions(&scan_params, regions);
+        if (scan_params.save_intermediate) {
+                struct bgr_image *marked;
+                marked = allocate_bgr_image8(height, width, NULL);
+                copy_bgr_image8(in, marked);
+                mark_regions(marked, regions);
+                colour_save_pnm("prunelarge.pnm", marked);
+                free(marked);
+        }
 	merge_regions(&scan_params, regions);
+        if (scan_params.save_intermediate) {
+                struct bgr_image *marked;
+                marked = allocate_bgr_image8(height, width, NULL);
+                copy_bgr_image8(in, marked);
+                mark_regions(marked, regions);
+                colour_save_pnm("merged.pnm", marked);
+                free(marked);
+        }
 	prune_small_regions(&scan_params, regions);
 
         if (scan_params.save_intermediate) {
