@@ -7,10 +7,12 @@ May 2012
 
 import numpy, os, cv, sys, cuav_util, time, math, functools, cuav_region
 
-from MAVProxy.modules.mavproxy_map import mp_image, mp_slipmap
+from MAVProxy.modules.lib import mp_image
+from MAVProxy.modules.mavproxy_map import mp_slipmap
 from cuav.image import scanner
 from cuav.camera.cam_params import CameraParams
 from MAVProxy.modules.lib.mp_menu import *
+from MAVProxy.modules.lib.wxsettings import WXSettings
 
 class MosaicRegion:
     def __init__(self, ridx, region, filename, pos, full_thumbnail, small_thumbnail, latlon=(None,None)):
@@ -24,6 +26,12 @@ class MosaicRegion:
         self.ridx = ridx
         self.score = region.score
 
+    def tag_image_available(self, color=(0,255,255)):
+        '''tag the small thumbnail image with a marker making it clear the
+        full image is available'''
+        (w,h) = cuav_util.image_shape(self.small_thumbnail)
+        cv.Rectangle(self.small_thumbnail, (w-3,0), (w-1,2), color, 2) 
+
     def __str__(self):
         position_string = ''
         if self.latlon != (None,None):
@@ -80,7 +88,10 @@ def ExtractThumbs(img, count):
 class Mosaic():
     '''keep a mosaic of found regions'''
     def __init__(self, slipmap,
-                 grid_width=20, grid_height=20, thumb_size=35, C=CameraParams()):
+                 grid_width=20, grid_height=20, thumb_size=35, C=CameraParams(),
+                 camera_settings = None,
+                 image_settings = None,
+                 start_menu=False):
         self.thumb_size = thumb_size
         self.width = grid_width * thumb_size
         self.height = grid_height * thumb_size
@@ -89,6 +100,7 @@ class Mosaic():
         self.display_regions = grid_width*grid_height
         self.regions = []
         self.regions_sorted = []
+        self.ridx_by_frame_time = {}
         self.page = 0
         self.sort_type = 'Time'
         self.images = []
@@ -100,39 +112,82 @@ class Mosaic():
         self.displayed_image = None
         self.last_click_position = None
         self.c_params = C
+        self.camera_settings = camera_settings
+        self.image_settings = image_settings
+        self.start_menu = start_menu
+        self.has_started = not start_menu
         import wx
         self.image_mosaic = mp_image.MPImage(title='Mosaic', 
                                              mouse_events=True,
                                              key_events=True,
-                                             auto_size=True)
+                                             auto_size=False,
+                                             report_size_changes=True)
         self.slipmap = slipmap
         self.selected_region = 0
 
         self.view_image = None
         self.brightness = 1
 
+        # dictionary of image requests, contains True if fullres image is wanted
+        self.image_requests = {}
+
         self.slipmap.add_callback(functools.partial(self.map_callback))
 
-        self.menu = MPMenuTop([MPMenuSubMenu('View',
-                                             items=[MPMenuRadio('Sort By', 'Select sorting key',
-                                                                returnkey='setSort',
-                                                                selected=self.sort_type,
-                                                                items=['Score\tAlt+S',
-                                                                       'Compactness\tAlt+C',
-                                                                       'Distinctiveness\tAlt+D',
-                                                                       'Whiteness\tAlt+W',
-                                                                       'Time\tAlt+T']),
-                                                    MPMenuItem('Next Page\tCtrl+N', 'Next Page', 'nextPage'),
-                                                    MPMenuItem('Previous Page\tCtrl+P', 'Previous Page', 'previousPage'),
-                                                    MPMenuItem('Brightness +\tCtrl+B', 'Increase Brightness', 'increaseBrightness'),
-                                                    MPMenuItem('Brightness -\tCtrl+Shift+B', 'Decrease Brightness', 'decreaseBrightness')
-                                                    ])])
+        self.add_menus()
+
+    def add_menus(self):
+        '''add menus'''
+        menu = MPMenuTop([])
+        if self.start_menu:
+            menu.add(MPMenuSubMenu('GEOSearch',
+                                   items=[MPMenuItem('Start', 'Start', 'menuStart'),
+                                          MPMenuItem('Stop', 'Stop', 'menuStop')]))
+        view_menu = MPMenuSubMenu('View',
+                                  items=[MPMenuRadio('Sort By', 'Select sorting key',
+                                                     returnkey='setSort',
+                                                     selected=self.sort_type,
+                                                     items=['Score\tAlt+S',
+                                                            'Compactness\tAlt+C',
+                                                            'Distinctiveness\tAlt+D',
+                                                            'Whiteness\tAlt+W',
+                                                            'Time\tAlt+T']),
+                                         MPMenuItem('Next Page\tCtrl+N', 'Next Page', 'nextPage'),
+                                         MPMenuItem('Previous Page\tCtrl+P', 'Previous Page', 'previousPage'),
+                                         MPMenuItem('Brightness +\tCtrl+B', 'Increase Brightness', 'increaseBrightness'),
+                                         MPMenuItem('Brightness -\tCtrl+Shift+B', 'Decrease Brightness', 'decreaseBrightness')
+                                         ])
+        menu.add(view_menu)
+        if self.camera_settings:
+            menu.add(MPMenuSubMenu('Camera',
+                                   items=[MPMenuItem('Settings', 'Settings', 'menuCameraSettings')]))
+        if self.image_settings:
+            menu.add(MPMenuSubMenu('Image',
+                                   items=[MPMenuItem('Settings', 'Settings', 'menuImageSettings')]))
+
+        self.menu = menu
         self.image_mosaic.set_menu(self.menu)
 
         self.popup_menu = MPMenuSubMenu('Popup',
-                                        items=[MPMenuItem('Show Image', returnkey='showImage')])
+                                        items=[MPMenuItem('Show Image', returnkey='showImage'),
+                                               MPMenuItem('Fetch Image', returnkey='fetchImage'),
+                                               MPMenuItem('Fetch Image (full)', returnkey='fetchImageFull'),
+                                               view_menu])
         self.image_mosaic.set_popup_menu(self.popup_menu)
 
+    def set_mosaic_size(self, size):
+        '''change mosaic size'''
+        (self.width, self.height) = size
+        grid_width = self.width // self.thumb_size
+        if grid_width < 1:
+            grid_width = 1
+        grid_height = self.height // self.thumb_size
+        if grid_height < 1:
+            grid_height = 1
+        ridx = self.page * self.display_regions
+        self.display_regions = grid_width * grid_height
+        self.page = ridx / self.display_regions
+        self.redisplay_mosaic()
+
     def set_brightness(self, b):
         '''set mosaic brightness'''
         self.brightness = b
@@ -239,8 +294,21 @@ class Mosaic():
         image = self.images[closest]
         self.view_imagefile(image.filename)
 
+    def map_menu_callback(self, event):
+        '''called on popup menu on map'''
+        menuitem = event.menuitem
+        if menuitem.returnkey == 'showImage':
+            region = self.objkey_to_region(event.selected[0].objkey)
+            self.popup_show_image(region)
+        elif menuitem.returnkey in ['fetchImage', 'fetchImageFull']:
+            region = self.objkey_to_region(event.selected[0].objkey)
+            self.popup_fetch_image(region, menuitem.returnkey)
+
     def map_callback(self, event):
         '''called when an event happens on the slipmap'''
+        if isinstance(event, mp_slipmap.SlipMenuEvent):
+            self.map_menu_callback(event)
+            return
         if not isinstance(event, mp_slipmap.SlipMouseEvent):
             return
         if event.event.m_middleDown:
@@ -314,12 +382,46 @@ class Mosaic():
             self.brightness /= 1.25
         elif event.returnkey == 'showImage':
             region = self.pos_to_region(event.popup_pos)
-            if region is not None:
-                self.show_region(region.ridx, True)
-                if region.latlon != (None,None):
-                    self.slipmap.add_object(mp_slipmap.SlipCenter(region.latlon))
+            self.popup_show_image(region)
+        elif event.returnkey in ['fetchImage', 'fetchImageFull']:
+            region = self.pos_to_region(event.popup_pos)
+            self.popup_fetch_image(region, event.returnkey)
+        elif event.returnkey == 'menuCameraSettings':
+            WXSettings(self.camera_settings)
+        elif event.returnkey == 'menuImageSettings':
+            WXSettings(self.image_settings)
+        elif event.returnkey == 'menuStart':
+            self.has_started = True
+        elif event.returnkey == 'menuStop':
+            self.has_started = False
         self.redisplay_mosaic()
 
+    def started(self):
+        '''return if start button has been pushed'''
+        return self.has_started
+
+    def popup_show_image(self, region):
+        '''handle popup menu showImage'''
+        if region is None:
+            return
+        self.show_region(region.ridx, True)
+        if region.latlon != (None,None):
+            self.slipmap.add_object(mp_slipmap.SlipCenter(region.latlon))
+
+    def popup_fetch_image(self, region, returnkey):
+        '''handle popup menu fetchImage'''
+        if region is None:
+            return
+        fullres = (returnkey == 'fetchImageFull')
+        frame_time = cuav_util.parse_frame_time(region.filename)
+        self.image_requests[frame_time] = fullres
+
+    def get_image_requests(self):
+        '''return and zero image_requests dictionary'''
+        ret = self.image_requests
+        self.image_requests = {}
+        return ret
+
     def menu_event_view(self, event):
         '''called on menu events on the view image'''
         if event.returnkey == 'increaseBrightness':
@@ -351,6 +453,15 @@ class Mosaic():
             return None
         return self.regions_sorted[ridx]
 
+    def objkey_to_region(self, objkey):
+        '''work out region for a map objkey'''
+        if not objkey.startswith("region "):
+            return None
+        ridx = int(objkey[7:])
+        if ridx < 0 or ridx >= len(self.regions):
+            return None
+        return self.regions[ridx]
+
     def mouse_event(self, event):
         '''called on mouse events on the mosaic'''
         # work out which region they want, taking into account wrap
@@ -384,19 +495,26 @@ class Mosaic():
     def display_mosaic_region(self, ridx):
         '''display a thumbnail on the mosaic'''
         region = self.regions_sorted[ridx]
+        width = (self.width // self.thumb_size) * self.thumb_size
         page_idx = ridx - self.page * self.display_regions
         if page_idx < 0 or page_idx >= self.display_regions:
             # its not on this page
             return
-        dest_x = (page_idx * self.thumb_size) % self.width
-        dest_y = ((page_idx * self.thumb_size) / self.width) * self.thumb_size
+        dest_x = (page_idx * self.thumb_size) % width
+        dest_y = ((page_idx * self.thumb_size) / width) * self.thumb_size
 
         # overlay thumbnail on mosaic
-        cuav_util.OverlayImage(self.mosaic, region.small_thumbnail, dest_x, dest_y)
+        #print dest_x, dest_y, self.width, self.height, self.thumb_size, cuav_util.image_width(region.small_thumbnail)
+        try:
+            cuav_util.OverlayImage(self.mosaic, region.small_thumbnail, dest_x, dest_y)
+        except Exception:
+            pass
 
     def redisplay_mosaic(self):
         '''re-display whole mosaic page'''
-        self.mosaic = cv.CreateImage((self.height,self.width),8,3)
+        width = (self.width // self.thumb_size) * self.thumb_size
+        height = (self.height // self.thumb_size) * self.thumb_size
+        self.mosaic = cv.CreateImage((width,height),8,3)
         cuav_util.zero_image(self.mosaic)
         for ridx in range(len(self.regions)):
             self.display_mosaic_region(ridx)
@@ -442,12 +560,19 @@ class Mosaic():
             self.regions.append(MosaicRegion(ridx, r, filename, pos, thumbs[i], thumb, latlon=(lat,lon)))
             self.regions_sorted.append(self.regions[-1])
 
+            frame_time = cuav_util.parse_frame_time(filename)
+            if not frame_time in self.ridx_by_frame_time:
+                self.ridx_by_frame_time[frame_time] = [ridx]
+            else:
+                self.ridx_by_frame_time[frame_time].append(ridx)
+
             self.display_mosaic_region(ridx)
 
             if (lat,lon) != (None,None):
                 self.slipmap.add_object(mp_slipmap.SlipThumbnail("region %u" % ridx, (lat,lon),
                                                                  img=thumb,
-                                                                 layer=2, border_width=1, border_colour=(255,0,0)))
+                                                                 layer=2, border_width=1, border_colour=(255,0,0),
+                                                                 popup_menu=self.popup_menu))
 
         self.image_mosaic.set_image(self.mosaic, bgr=True)
 
@@ -455,16 +580,28 @@ class Mosaic():
         '''add a camera image'''
         self.images.append(MosaicImage(frame_time, filename, pos))
 
+    def tag_image(self, frame_time, tag_color=(0,255,255)):
+        '''tag a mosaic image'''
+        if frame_time in self.ridx_by_frame_time:
+            for ridx in self.ridx_by_frame_time[frame_time]:
+                self.regions[ridx].tag_image_available(color=tag_color)
+                self.display_mosaic_region(ridx)
+
     def check_events(self):
         '''check for mouse/keyboard events'''
         if self.image_mosaic.is_alive():
             for event in self.image_mosaic.events():
-                if isinstance(event, MPMenuGeneric):
+                if isinstance(event, mp_image.MPImageNewSize):
+                    self.set_mosaic_size(event.size)
+                elif isinstance(event, MPMenuGeneric):
                     self.menu_event(event)
                 elif event.ClassName == 'wxMouseEvent':
                     self.mouse_event(event)
                 elif event.ClassName == 'wxKeyEvent':
                     self.key_event(event)
+                else:
+                    print('unknown event ', event)
+                    print(dir(event))
         if self.view_image and self.view_image.is_alive():
             for event in self.view_image.events():
                 if isinstance(event, MPMenuGeneric):
