@@ -450,8 +450,10 @@ def mkdir_p(dir):
 
 def frame_time(t):
     '''return a time string for a filename with 0.01 sec resolution'''
+    # round to the nearest 100th of a second
+    t += 0.005
     hundredths = int(t * 100.0) % 100
-    return "%s%02u" % (time.strftime("%Y%m%d%H%M%S", time.localtime(t)), hundredths)
+    return "%s%02uZ" % (time.strftime("%Y%m%d%H%M%S", time.gmtime(t)), hundredths)
 
 def parse_frame_time(filename):
 	'''parse a image frame time from a image filename
@@ -464,13 +466,22 @@ def parse_frame_time(filename):
                         return mav_position.exif_timestamp(filename)
 		raise RuntimeError('unable to parse filename %s into time' % basename)
 	tstring = basename[i:]
-	t = time.mktime(time.strptime(tstring[:14], "%Y%m%d%H%M%S"))
+        ttuple = time.strptime(tstring[:14], "%Y%m%d%H%M%S")
+	t = time.mktime(ttuple)
+        dst = time.localtime().tm_isdst
 	# hundredths can be after a dash
 	if tstring[14] == '-':
 		hundredths = int(tstring[15:17])
+                z = tstring[17]
 	else:
 		hundredths = int(tstring[14:16])
+                z = tstring[16]
 	t += hundredths * 0.01
+        if z.upper() == 'Z':
+                if dst > 0:
+                        t -= time.altzone
+                else:
+                        t -= time.timezone
 	return t
 
 
@@ -745,3 +756,33 @@ def zero_image(img):
 	'''zero an image'''
         cv.ConvertScale(img, img, scale=0)
 
+
+def set_system_clock(time_seconds):
+    '''sync system clock with GPS time
+    Thanks to tMC http://stackoverflow.com/questions/12081310/python-module-to-change-system-date-and-time
+    '''
+    import ctypes
+    import ctypes.util
+
+    # define CLOCK_REALTIME 0
+    CLOCK_REALTIME = 0
+
+    # /usr/include/time.h
+    #
+    # struct timespec
+    #  {
+    #    __time_t tv_sec;            /* Seconds.  */
+    #    long int tv_nsec;           /* Nanoseconds.  */
+    #  };
+    class timespec(ctypes.Structure):
+        _fields_ = [("tv_sec", ctypes.c_long), # hmm, is c_long right? what about 64bit time_t?
+                    ("tv_nsec", ctypes.c_long)]
+        
+    librt = ctypes.CDLL(ctypes.util.find_library("rt"))
+
+    ts = timespec()
+    ts.tv_sec = int(time_seconds)
+    ts.tv_nsec = int((time_seconds - int(time_seconds))*1e9)
+
+    # http://linux.die.net/man/3/clock_settime
+    return librt.clock_settime(CLOCK_REALTIME, ctypes.byref(ts))
