@@ -13,7 +13,7 @@ from cuav.image import scanner
 from pymavlink import mavutil
 from cuav.lib import cuav_mosaic, mav_position, cuav_util, cuav_joe, block_xmit, cuav_region
 from MAVProxy.modules.lib import mp_settings
-from MAVProxy.modules.mavproxy_map import mp_image
+from MAVProxy.modules.lib import mp_image
 from cuav.camera.cam_params import CameraParams
 from MAVProxy.modules.mavproxy_map import mp_slipmap
 
@@ -31,32 +31,37 @@ class MavSocket:
         self.incoming = []
 
     def sendto(self, buf, dest):
+        dbuf = [ord(x) for x in buf]
+        dbuf.extend([0]*(96-len(dbuf)))
         if len(buf) <= 16:
-            self.master.mav.data16_send(0, len(buf), buf)
+            self.master.mav.data16_send(0, len(buf), dbuf)
         elif len(buf) <= 32:
-            self.master.mav.data32_send(0, len(buf), buf)
+            self.master.mav.data32_send(0, len(buf), dbuf)
         elif len(buf) <= 64:
-            self.master.mav.data64_send(0, len(buf), buf)
+            self.master.mav.data64_send(0, len(buf), dbuf)
         elif len(buf) <= 96:
-            self.master.mav.data96_send(0, len(buf), buf)
+            self.master.mav.data96_send(0, len(buf), dbuf)
         else:
-            print("PACKET TOO LARGE %u" % len(buf))
-            raise RuntimeError('packet too large %u' % len(buf))
+            print("PACKET TOO LARGE %u" % len(dbuf))
+            raise RuntimeError('packet too large %u' % len(dbuf))
 
     def recvfrom(self, size):
         if len(self.incoming) == 0:
             return ('', 'mavlink')
         m = self.incoming.pop(0)
-        buf = bytes(m.data[:m.len])
+        data = m.data[:m.len]
+        s = ''.join([chr(x) for x in data])
+        buf = bytes(s)
         return (buf, 'mavlink')
 
 class ImagePacket:
     '''a jpeg image sent to the ground station'''
-    def __init__(self, frame_time, jpeg, xmit_queue, pos):
+    def __init__(self, frame_time, jpeg, xmit_queue, pos, priority):
         self.frame_time = frame_time
         self.jpeg = jpeg
         self.xmit_queue = xmit_queue
         self.pos = pos
+        self.priority = priority
 
 class ThumbPacket:
     '''a thumbnail region sent to the ground station'''
@@ -79,6 +84,24 @@ class CommandResponse:
         self.response = response
 
 
+class ImageRequest:
+    '''request a jpeg image from the aircraft'''
+    def __init__(self, frame_time, fullres):
+        self.frame_time = frame_time
+        self.fullres = fullres
+
+class ChangeCameraSetting:
+    '''update a camera setting'''
+    def __init__(self, name, value):
+        self.name = name
+        self.value = value
+
+class ChangeImageSetting:
+    '''update a image setting'''
+    def __init__(self, name, value):
+        self.name = name
+        self.value = value
+        
 
 class CameraModule(mp_module.MPModule):
     def __init__(self, mpstate):
@@ -95,37 +118,61 @@ class CameraModule(mp_module.MPModule):
         self.transmit_thread_h = None
         self.view_thread_h = None
 
-        self.camera_settings = mp_settings.MPSettings(
-            [ ('depth', int, 8),
-              ('gcs_address', str, None),
-              ('gcs_view_port', int, 7543),
-              ('bandwidth',  int, 40000),
-              ('bandwidth2', int, 2000),
-              ('capture_brightness', int, 150),
-              ('gamma', int, 950),
-              ('brightness', float, 1.0),
-              ('quality', int, 75),
-              ('save_pgm', int, 1),
-              ('transmit', int, 1),
-              ('roll_stabilised', int, 1),
-              ('minscore', int, 75),
-              ('minscore2', int, 500),
-              ('altitude', int, None),
-              ('send1', int, 1),
-              ('send2', int, 1),
-              ('maxqueue1', int, None),
-              ('maxqueue2', int, 30),
-              ('thumbsize', int, 60),
-              ('packet_loss', int, 0),             
-              ('gcs_slave', str, None),
-              ('filter_type', str, 'simple'),
-              ('fullres', int, 0),
-              ('use_bsend2', int, 1),
-              ('framerate', int, 7)  
-              ]
+        from MAVProxy.modules.lib.mp_settings import MPSettings, MPSetting
+        self.camera_settings = MPSettings(
+            [ MPSetting('depth', int, 8, 'Image Depth', choice=['8', '16'], tab='Capture'),
+              MPSetting('save_pgm', bool, True, 'Save Raw Images'),
+              MPSetting('capture_brightness', int, 150, 'Capture Brightness', range=(10, 300), increment=1),
+              MPSetting('gamma', int, 950, 'Capture Gamma', range=(0,1000), increment=1),
+              MPSetting('roll_stabilised', bool, True, 'Roll Stabilised'),
+              MPSetting('altitude', int, 0, 'Altitude', range=(0,10000), increment=1),
+              MPSetting('filter_type', str, 'simple', 'Filter Type',
+                        choice=['simple', 'compactness']),
+              MPSetting('fullres', bool, False, 'Full Resolution'),
+              MPSetting('framerate', str, 7, 'Frame Rate', choice=['1', '3', '7', '15']),
+              MPSetting('process_divider', int, 1, 'Process Divider', range=(1,50), increment=1),
+              MPSetting('use_capture_time', bool, False, 'Use Capture Time'),
+
+              MPSetting('gcs_address', str, None, 'GCS Address', tab='GCS'),
+              MPSetting('gcs_view_port', int, 7543, 'GCS View Port', range=(1, 30000), increment=1),
+              MPSetting('gcs_slave', str, None, 'GCS Slave'),
+              
+              MPSetting('bandwidth',  int, 40000, 'Link1 Bandwdith', 'Comms'),
+              MPSetting('bandwidth2', int, 2000, 'Link2 Bandwidth'),
+              MPSetting('quality', int, 75, 'Compression Quality', range=(1,100), increment=1),
+              MPSetting('transmit', bool, True, 'Transmit Enable'),
+              MPSetting('send1', bool, True, 'Send on Link1'),
+              MPSetting('send2', bool, True, 'Send on Link2'),
+              MPSetting('maxqueue1', int, None, 'Maximum queue Link1'),
+              MPSetting('maxqueue2', int, 30, 'Maxqueue queue Link2'),
+              MPSetting('thumbsize', int, 60, 'Thumbnail Size', range=(10, 200), increment=1),
+              MPSetting('mosaic_thumbsize', int, 35, 'Mosaic Thumbnail Size', range=(10, 200), increment=1),
+              MPSetting('use_bsend2', bool, True, 'Enable Link2'),
+
+              MPSetting('minscore', int, 75, 'Min Score Link1', range=(0,1000), increment=1, tab='Scoring'),
+              MPSetting('minscore2', int, 500, 'Min Score Link2', range=(0,1000), increment=1),
+              MPSetting('packet_loss', int, 0, 'Packet Loss', range=(0,100), increment=1, tab='Misc'),             
+              MPSetting('clock_sync', bool, False, 'GPS Clock Sync'),             
+
+              MPSetting('brightness', float, 1.0, 'Display Brightness', range=(0.1, 10), increment=0.1,
+                        digits=2, tab='Display')
+              ],
+            title='Camera Settings'
             )
 
+        self.image_settings = MPSettings(
+            [ MPSetting('MinRegionArea', float, 0.15, range=(0,100), increment=0.05, digits=2, tab='Image Processing'),
+              MPSetting('MaxRegionArea', float, 2.0, range=(0,100), increment=0.1, digits=1),
+              MPSetting('MinRegionSize', float, 0.1, range=(0,100), increment=0.05, digits=2),
+              MPSetting('MaxRegionSize', float, 2, range=(0,100), increment=0.1, digits=1),
+              MPSetting('MaxRarityPct',  float, 0.02, range=(0,100), increment=0.01, digits=2),
+              MPSetting('RegionMergeSize', float, 3.0, range=(0,100), increment=0.1, digits=1),
+              MPSetting('SaveIntermediate', bool, False)
+              ],
+            title='Image Settings')
+
         self.capture_count = 0
+        self.process_counter = 0
         self.scan_count = 0
         self.error_count = 0
         self.error_msg = None
@@ -137,6 +184,7 @@ class CameraModule(mp_module.MPModule):
         self.scan_queue = Queue.Queue()
         self.transmit_queue = Queue.Queue()
         self.viewing = False
+        self.have_set_gps_time = False
         
         self.c_params = CameraParams(lens=4.0)
         self.jpeg_size = 0
@@ -182,7 +230,7 @@ class CameraModule(mp_module.MPModule):
 
     def cmd_camera(self, args):
         '''camera commands'''
-        usage = "usage: camera <start|stop|status|view|noview|boundary|set>"
+        usage = "usage: camera <start|stop|status|view|noview|boundary|set|image>"
         if len(args) == 0:
             print(usage)
             return
@@ -208,9 +256,9 @@ class CameraModule(mp_module.MPModule):
                 self.region_count, 
                 self.jpeg_size,
                 self.xmit_queue, self.xmit_queue2, self.frame_loss, self.scan_queue.qsize(), self.efficiency))
-            # print("self.bsend2 is ", self.bsend2)
+            print("self.bsend2 is ", self.bsend2)
             if self.bsend2 is not None:
-                self.bsend2.report(detailed=False)
+                self.bsend2.report(detailed=True)
         elif args[0] == "queue":
             print("scan %u  save %u  transmit %u  eff %.1f  bw %.1f  rtt %.1f" % (
                 self.scan_queue.qsize(),
@@ -234,6 +282,8 @@ class CameraModule(mp_module.MPModule):
             self.viewing = False
         elif args[0] == "set":
             self.camera_settings.command(args[1:])
+        elif args[0] == "image":
+            self.image_settings.command(args[1:])
         elif args[0] == "boundary":
             if len(args) != 2:
                 print("boundary=%s" % self.boundary)
@@ -251,19 +301,7 @@ class CameraModule(mp_module.MPModule):
         '''camera commands'''
         cmd = " ".join(args)
         pkt = CommandPacket(cmd)
-        buf = cPickle.dumps(pkt, cPickle.HIGHEST_PROTOCOL)
-        self.start_gcs_bsend()
-        if self.camera_settings.use_bsend2:
-            if self.bsend2 is None:
-                print("bsend2 not initialised")
-                return
-            self.bsend2.set_bandwidth(self.camera_settings.bandwidth2)
-            self.bsend2.send(buf, priority=10000)
-        else:
-            if self.bsend is None:
-                print("bsend not initialised")
-                return
-            self.bsend.send(buf, priority=10000)
+        self.send_packet(pkt)
 
     def get_base_time(self):
         '''we need to get a baseline time from the camera. To do that we trigger
@@ -322,6 +360,7 @@ class CameraModule(mp_module.MPModule):
             try:
                 if h is None:
                     h, base_time, last_frame_time = self.get_base_time()
+                    last_capture_frame_time = last_frame_time
                     # put into continuous mode
                     chameleon.trigger(h, True)
 
@@ -333,25 +372,38 @@ class CameraModule(mp_module.MPModule):
                 if last_gamma != self.camera_settings.gamma:
                     chameleon.set_gamma(h, self.camera_settings.gamma)
                     last_gamma = self.camera_settings.gamma
-                if last_framerate != self.camera_settings.framerate:
-                    chameleon.set_framerate(h, self.camera_settings.framerate)
-                    last_framerate = self.camera_settings.framerate
+                if last_framerate != int(self.camera_settings.framerate):
+                    chameleon.set_framerate(h, int(self.camera_settings.framerate))
+                    last_framerate = int(self.camera_settings.framerate)
+
+                # capture an image
                 frame_time, frame_counter, shutter = chameleon.capture(h, 1000, im)
-                if frame_time < last_frame_time:
+                if frame_time < last_capture_frame_time:
                     base_time += 128
+                last_capture_frame_time = frame_time
                 if last_frame_counter != 0:
                     self.frame_loss += frame_counter - (last_frame_counter+1)
+
+                # discard based on process_divider setting
+                self.process_counter = (self.process_counter + 1) % self.camera_settings.process_divider
+                if self.process_counter % self.camera_settings.process_divider != 0:
+                    continue
                 
+                if self.camera_settings.use_capture_time:
+                    img_time = capture_time
+                else:
+                    img_time = base_time + frame_time
+
                 gammalog.write('%f %f %f %s %u %u\n' % (frame_time,
                                                         frame_time+base_time,
                                                         capture_time,
-                                                        cuav_util.frame_time(frame_time+base_time),
+                                                        cuav_util.frame_time(img_time),
                                                         frame_counter,
                                                         self.camera_settings.gamma))
                 gammalog.flush()
 
-                self.save_queue.put((base_time+frame_time,im))
-                self.scan_queue.put((base_time+frame_time,im))
+                self.save_queue.put((img_time,im))
+                self.scan_queue.put((img_time,im))
                 self.capture_count += 1
                 self.fps = 1.0/(frame_time - last_frame_time)
 
@@ -391,6 +443,11 @@ class CameraModule(mp_module.MPModule):
             except Queue.Empty:
                 continue
 
+            scan_parms = {}
+            for name in self.image_settings.list():
+                scan_parms[name] = self.image_settings.get(name)
+            scan_parms['SaveIntermediate'] = float(scan_parms['SaveIntermediate'])
+            
             t1 = time.time()
             im_full = numpy.zeros((960,1280,3),dtype='uint8')
             im_640 = numpy.zeros((480,640,3),dtype='uint8')
@@ -433,8 +490,11 @@ class CameraModule(mp_module.MPModule):
     def log_joe_position(self, pos, frame_time, regions, filename=None, thumb_filename=None):
         '''add to joe.log if possible, returning a list of (lat,lon) tuples
         for the positions of the identified image regions'''
+        altitude = self.camera_settings.altitude
+        if altitude <= 0:
+            altitude = None
         return self.joelog.add_regions(frame_time, regions, pos, filename,
-                                       thumb_filename, altitude=self.camera_settings.altitude)
+                                       thumb_filename, altitude=altitude)
 
 
     def transmit_thread(self):
@@ -520,27 +580,29 @@ class CameraModule(mp_module.MPModule):
 
             # Base how many images we send on the send queue size
             send_frequency = self.xmit_queue // 3
-            if send_frequency == 0 or (tx_count+skip_count) % send_frequency == 0:
-                jpeg = scanner.jpeg_compress(im_640, self.camera_settings.quality)
-
-            if jpeg is None:
-                skip_count += 1
-                continue
-
-            # keep filtered image size
-            self.jpeg_size = 0.95 * self.jpeg_size + 0.05 * len(jpeg)
-        
-            tx_count += 1
-
             if self.camera_settings.gcs_address is None:
                 continue
-            self.bsend.set_packet_loss(self.camera_settings.packet_loss)
-            self.bsend.set_bandwidth(self.camera_settings.bandwidth)
-            pkt = ImagePacket(frame_time, jpeg, self.xmit_queue, pos)
-            str = cPickle.dumps(pkt, cPickle.HIGHEST_PROTOCOL)
-            # print("sending image len=%u" % len(str))
-            self.bsend.send(str,
-                            dest=(self.camera_settings.gcs_address, self.camera_settings.gcs_view_port))
+            if send_frequency != 0 and (tx_count+skip_count) % send_frequency != 0:
+                skip_count += 1
+                continue
+            tx_count += 1
+            self.send_image(im_640, frame_time, pos, 0)
+
+    def send_image(self, img, frame_time, pos, priority):
+        '''send an image to the GCS'''
+        jpeg = scanner.jpeg_compress(img, self.camera_settings.quality)
+
+        # keep filtered image size
+        self.jpeg_size = 0.95 * self.jpeg_size + 0.05 * len(jpeg)        
+
+        self.bsend.set_packet_loss(self.camera_settings.packet_loss)
+        self.bsend.set_bandwidth(self.camera_settings.bandwidth)
+        pkt = ImagePacket(frame_time, jpeg, self.xmit_queue, pos, priority)
+        str = cPickle.dumps(pkt, cPickle.HIGHEST_PROTOCOL)
+        # print("sending image len=%u" % len(str))
+        self.bsend.send(str,
+                        dest=(self.camera_settings.gcs_address, self.camera_settings.gcs_view_port),
+                        priority=priority)
 
 
     def reload_mosaic(self, mosaic):
@@ -559,7 +621,6 @@ class CameraModule(mp_module.MPModule):
                 try:
                     composite = cv.LoadImage(last_joe.thumb_filename)
                     thumbs = cuav_mosaic.ExtractThumbs(composite, len(regions))
-                    mosaic.set_brightness(self.camera_settings.brightness)
                     mosaic.add_regions(regions, thumbs, last_joe.image_filename, last_joe.pos)
                 except Exception:
                     pass                
@@ -570,7 +631,6 @@ class CameraModule(mp_module.MPModule):
             try:
                 composite = cv.LoadImage(last_joe.thumb_filename)
                 thumbs = cuav_mosaic.ExtractThumbs(composite, len(regions))
-                mosaic.set_brightness(self.camera_settings.brightness)
                 mosaic.add_regions(regions, thumbs, last_joe.image_filename, last_joe.pos)
             except Exception:
                 pass
@@ -597,7 +657,6 @@ class CameraModule(mp_module.MPModule):
             self.bsend2.set_bandwidth(self.camera_settings.bandwidth2)
 
 
-
     def view_thread(self):
         '''image viewing thread - this runs on the ground station'''
         from cuav.lib import cuav_mosaic
@@ -630,6 +689,9 @@ class CameraModule(mp_module.MPModule):
 
         ack_time = time.time()
 
+        self.camera_settings.set_callback(self.camera_settings_callback)
+        self.image_settings.set_callback(self.image_settings_callback)
+
         while not self.unload_event.wait(0.02):
             if not self.viewing:
                 if view_window:
@@ -645,7 +707,10 @@ class CameraModule(mp_module.MPModule):
                     ack_time = tnow
             if not view_window:
                 view_window = True
-                mosaic = cuav_mosaic.Mosaic(slipmap=self.mpstate.map, C=self.c_params)
+                mosaic = cuav_mosaic.Mosaic(slipmap=self.mpstate.map, C=self.c_params,
+                                            camera_settings=self.camera_settings,
+                                            image_settings=self.image_settings,
+                                            thumb_size=self.camera_settings.mosaic_thumbsize)
                 if self.boundary_polygon is not None:
                     mosaic.set_boundary(self.boundary_polygon)
                 if self.continue_mode:
@@ -654,6 +719,8 @@ class CameraModule(mp_module.MPModule):
             # check for keyboard events
             mosaic.check_events()
 
+            self.check_requested_images(mosaic)
+
             buf = self.bsend.recv(0)
             if buf is None:
                 buf = self.bsend2.recv(0)
@@ -699,7 +766,6 @@ class CameraModule(mp_module.MPModule):
                 self.log_joe_position(pos, obj.frame_time, obj.regions, filename, thumb_filename)
 
                 # update the mosaic and map
-                mosaic.set_brightness(self.camera_settings.brightness)
                 mosaic.add_regions(obj.regions, thumbs, filename, pos=pos)
 
                 # update console display
@@ -731,8 +797,17 @@ class CameraModule(mp_module.MPModule):
                 else:
                     display_img = img
 
-                mosaic.add_image(obj.frame_time, filename, obj.pos)
+                if obj.pos is not None:
+                    mosaic.add_image(obj.frame_time, filename, obj.pos)
 
+                if obj.priority != 0:
+                    print("Downloaded image %s (width %u)" % (filename, img.width))
+                    if img.width >= 1280:
+                        tag_color = (0,0,255)
+                    else:
+                        tag_color = (0,255,0)
+                    mosaic.tag_image(obj.frame_time, tag_color=tag_color)
+                    
                 cv.ConvertScale(display_img, display_img, scale=self.camera_settings.brightness)
                 img_window.set_image(display_img, bgr=True)
 
@@ -746,10 +821,9 @@ class CameraModule(mp_module.MPModule):
 
             if isinstance(obj, CommandPacket):
                 self.handle_command_packet(obj, bsend)
-            
+
             if isinstance(obj, CommandResponse):
                 print('REMOTE: %s' % obj.response)
-                
 
     def start_thread(self, fn):
         '''start a thread running'''
@@ -777,7 +851,7 @@ class CameraModule(mp_module.MPModule):
         stdout_saved = sys.stdout
         buf = cStringIO.StringIO()
         sys.stdout = buf
-        self.mpstate.functions.process_stdin(obj.command)
+        self.mpstate.functions.process_stdin(obj.command, immediate=True)
         sys.stdout = stdout_saved
         pkt = CommandResponse(str(buf.getvalue()))
         buf = cPickle.dumps(pkt, cPickle.HIGHEST_PROTOCOL)
@@ -803,6 +877,15 @@ class CameraModule(mp_module.MPModule):
         if isinstance(obj, CommandResponse):
             print('REMOTE: %s' % obj.response)
 
+        if isinstance(obj, ImageRequest):
+            self.handle_image_request(obj, bsend)            
+
+        if isinstance(obj, ChangeCameraSetting):
+            self.camera_settings.set(obj.name, obj.value)
+
+        if isinstance(obj, ChangeImageSetting):
+            self.image_settings.set(obj.name, obj.value)
+
     def mavlink_packet(self, m):
         '''handle an incoming mavlink packet'''
         if self.mpstate.status.watch in ["camera","queue"] and time.time() > self.last_watch+1:
@@ -813,6 +896,94 @@ class CameraModule(mp_module.MPModule):
         if m.get_type() in [ 'DATA16', 'DATA32', 'DATA64', 'DATA96' ]:
             if self.bsocket is not None:
                 self.bsocket.incoming.append(m)
+        if m.get_type() == 'SYSTEM_TIME' and self.camera_settings.clock_sync and self.capture_thread_h is not None:
+            # optionally sync system clock on the capture side
+            self.sync_gps_clock(m.time_unix_usec)
+
+    def sync_gps_clock(self, time_usec):
+        '''sync system clock with GPS time'''
+        if time_usec == 0:
+            # no GPS lock
+            return
+        if os.geteuid() != 0:
+            # can only do this as root
+            return
+        time_seconds = time_usec*1.0e-6
+        if self.have_set_gps_time and abs(time_seconds - time.time()) < 10:
+            # only change a 2nd time if time is off by 10 seconds
+            return
+        t1 = time.time()
+        cuav_util.set_system_clock(time_seconds)
+        t2 = time.time()
+        print("Changed system time by %.2f seconds" % (t2-t1))
+        self.have_set_gps_time = True
+
+    def check_requested_images(self, mosaic):
+        '''check if the user has requested download of an image'''
+        requests = mosaic.get_image_requests()
+        for frame_time in requests.keys():
+            fullres = requests[frame_time]
+            pkt = ImageRequest(frame_time, fullres)
+            buf = cPickle.dumps(pkt, cPickle.HIGHEST_PROTOCOL)
+            self.start_gcs_bsend()
+            print("Requesting image %s" % frame_time)
+            if self.camera_settings.use_bsend2:
+                if self.bsend2 is None:
+                    print("bsend2 not initialised")
+                    return
+                self.bsend2.set_bandwidth(self.camera_settings.bandwidth2)
+                self.bsend2.send(buf, priority=10000)
+            else:
+                if self.bsend is None:
+                    print("bsend not initialised")
+                    return
+                self.bsend.send(buf, priority=10000)
+
+    def handle_image_request(self, obj, bsend):
+        '''handle ImageRequest from GCS'''
+        rawname = "raw%s" % cuav_util.frame_time(obj.frame_time)
+        raw_dir = os.path.join(self.camera_dir, "raw")
+        filename = '%s/%s.pgm' % (raw_dir, rawname)
+        if not os.path.exists(filename):
+            print("No file: %s" % filename)
+            return
+        try:
+            img = cuav_util.LoadImage(filename)
+            img = numpy.asarray(cv.GetMat(img))
+        except Exception:
+            return
+        if not obj.fullres:
+            im_640 = numpy.zeros((480,640,3),dtype='uint8')
+            scanner.downsample(img, im_640)
+            img = im_640
+        print("Sending image %s" % filename)
+        self.send_image(img, obj.frame_time, None, 10000)
+
+    def send_packet(self, pkt):
+        '''send a packet from GCS'''
+        buf = cPickle.dumps(pkt, cPickle.HIGHEST_PROTOCOL)
+        self.start_gcs_bsend()
+        if self.camera_settings.use_bsend2:
+            if self.bsend2 is None:
+                print("bsend2 not initialised")
+                return
+            self.bsend2.set_bandwidth(self.camera_settings.bandwidth2)
+            self.bsend2.send(buf, priority=10000)
+        else:
+            if self.bsend is None:
+                print("bsend not initialised")
+                return
+            self.bsend.send(buf, priority=10000)
+
+    def camera_settings_callback(self, setting):
+        '''called on a changed camera setting'''
+        pkt = ChangeCameraSetting(setting.name, setting.value)
+        self.send_packet(pkt)
+
+    def image_settings_callback(self, setting):
+        '''called on a changed image setting'''
+        pkt = ChangeImageSetting(setting.name, setting.value)
+        self.send_packet(pkt)
 
 def init(mpstate):
     '''initialise module'''
